import { AStar } from '#application/character/aStar'
import ZoneManager from '#managers/zoneManager'
import Rotation from '#application/character/rotation'
import { appLogger, gameLogger } from '#application/logger'
import { Database } from '#application/database'
import { Character } from '#entities/character'
import UserRepository from '#repositories/userRepository'
import CharacterRepository from '#repositories/characterRepository'
import CharacterHairRepository from '#repositories/characterHairRepository'
import ZoneRepository from '#repositories/zoneRepository'
import { Zone } from '#entities/zone'

interface Position {
  x: number
  y: number
}

export class CharacterService {
  private readonly MOVEMENT_DELAY_MS = 250

  async updateCharacterPosition(id: number, positionX: number, positionY: number, rotation: number, zoneId: number) {
    const character = await CharacterRepository.getById(id)
    if (!character) return null

    character
      .setPositionX(positionX)
      .setPositionY(positionY)
      .setRotation(rotation)
      .setZone((await ZoneRepository.getById(zoneId)) as Zone)

    await character.save()

    return character
  }

  public async calculatePath(character: Character, targetX: number, targetY: number): Promise<Position[] | null> {
    const zone = ZoneManager.getZoneById(character.zone!.id)
    const grid = await zone?.getGrid()

    if (!grid?.length) {
      gameLogger.error('character:move error', 'Grid not found or empty')
      return null
    }

    const start: Position = {
      x: Math.floor(character.positionX),
      y: Math.floor(character.positionY)
    }

    const end: Position = {
      x: Math.floor(targetX),
      y: Math.floor(targetY)
    }

    return AStar.findPath(start, end, grid)
  }

  public async applyMovementDelay(): Promise<void> {
    await new Promise((resolve) => setTimeout(resolve, this.MOVEMENT_DELAY_MS))
  }

  private isValidPosition(position: Position): boolean {
    return Number.isFinite(position.x) && Number.isFinite(position.y) && position.x >= 0 && position.y >= 0
  }
}
